@@ -14,7 +14,8 @@
 #undef TRACE_SYSTEM
 #define TRACE_SYSTEM kvm_mmu
 
-#if (LINUX_VERSION_CODE >= KERNEL_VERSION(5,1,0))
+#if (LINUX_VERSION_CODE >= KERNEL_VERSION(5,1,0) || \
+	LTTNG_RHEL_KERNEL_RANGE(4,18,0,147,0,0, 4,19,0,0,0,0))
 
 #define LTTNG_KVM_MMU_PAGE_FIELDS \
 	ctf_integer(__u64, gfn, (sp)->gfn) \
@@ -217,7 +218,11 @@ LTTNG_TRACEPOINT_EVENT_MAP(
 #if (LINUX_VERSION_CODE >= KERNEL_VERSION(5,6,0) || \
 	LTTNG_KERNEL_RANGE(4,19,103, 4,20,0) || \
 	LTTNG_KERNEL_RANGE(5,4,19, 5,5,0) || \
-	LTTNG_KERNEL_RANGE(5,5,3, 5,6,0))
+	LTTNG_KERNEL_RANGE(5,5,3, 5,6,0) || \
+	LTTNG_UBUNTU_KERNEL_RANGE(4,15,18,92, 4,16,0,0) || \
+	LTTNG_UBUNTU_KERNEL_RANGE(5,0,21,44, 5,1,0,0) || \
+	LTTNG_UBUNTU_KERNEL_RANGE(5,3,18,43, 5,3,18,45) || \
+	LTTNG_UBUNTU_KERNEL_RANGE(5,3,18,46, 5,4,0,0))
 LTTNG_TRACEPOINT_EVENT_MAP(
 	fast_page_fault,
 
