@@ -4,12 +4,16 @@
  *
  * wrapper around irq_to_desc. Using KALLSYMS to get its address when
  * available, else we need to have a kernel that exports this function to GPL
- * modules.
+ * modules. This export was added to the 3.4 kernels.
  *
  * Copyright (C) 2011-2012 Mathieu Desnoyers <mathieu.desnoyers@efficios.com>
  */
 
-#ifdef CONFIG_KALLSYMS
+#include <lttng-kernel-version.h>
+#include <linux/module.h>
+
+#if (defined(CONFIG_KALLSYMS) \
+	&& (LINUX_VERSION_CODE < KERNEL_VERSION(3,4,0)))
 
 #include <linux/kallsyms.h>
 #include <linux/interrupt.h>
@@ -31,6 +35,7 @@ struct irq_desc *wrapper_irq_to_desc(unsigned int irq)
 		return NULL;
 	}
 }
+EXPORT_SYMBOL_GPL(wrapper_irq_to_desc);
 
 #else
 
@@ -41,5 +46,6 @@ struct irq_desc *wrapper_irq_to_desc(unsigned int irq)
 {
 	return irq_to_desc(irq);
 }
+EXPORT_SYMBOL_GPL(wrapper_irq_to_desc);
 
 #endif
