@@ -17,4 +17,17 @@
 
 struct irq_desc *wrapper_irq_to_desc(unsigned int irq);
 
+/*
+ * Canary function to check for 'irq_to_desc()' at compile time.
+ *
+ * From 'include/linux/irqnr.h':
+ *
+ *   extern struct irq_desc *irq_to_desc(unsigned int irq);
+ */
+static inline
+struct irq_desc *__canary__irq_to_desc(unsigned int irq)
+{
+	return irq_to_desc(irq);
+}
+
 #endif /* _LTTNG_WRAPPER_IRQDESC_H */
